package geniusweb.boa.biddingstrategy;

import geniusweb.actions.Accept;
import geniusweb.actions.Action;
import geniusweb.actions.EndNegotiation;
import geniusweb.actions.Offer;
import geniusweb.boa.BoaParty;
import geniusweb.boa.BoaState;
import geniusweb.boa.acceptancestrategy.AcceptanceStrategy;
import geniusweb.inform.Settings;

/**
 * Determines what action to take in which state. The protocol to be supported
 * is SAOP.
 * <p>
 * Assumed part of a {@link BoaParty}.
 * 
 * <p>
 * <em>MUST</em> have an empty constructor.
 * <h2>More info</h2> Baarslag T., Hindriks K.V., Hendrikx M., Dirkzwager A.,
 * Jonker C.M. Decoupling Negotiating Agents to Explore the Space of Negotiation
 * Strategies. Proceedings of The Fifth International Workshop on Agent-based
 * Complex Automated Negotiations (ACAN 2012), 2012.
 * ​https://homepages.cwi.nl/~baarslag/pub/Decoupling_Negotiating_Agents_to_Explore_the_Space_of_Negotiation_Strategies_ACAN_2012.pdf
 */
public interface BiddingStrategy {

	/**
	 * Recommend a next action.
	 * <p>
	 * <b>Note</b> for implementors: This can extract configuration parameters
	 * from {@link Settings#getParameters} using {@link BoaState#getSettings()}.
	 * Generally we recommend to write implementations such that they can also
	 * be hard-configured, for easier use of the components in hard-coded
	 * settings.
	 * 
	 * @param state the current {@link BoaState}
	 * 
	 * @return the suggested next Action. Should always return a valid action,
	 *         not null. Typically this generates only {@link Offer}s (and maybe
	 *         {@link EndNegotiation}. The subsequent {@link AcceptanceStrategy}
	 *         will replace {@link Offer}s with {@link Accept}s as needed.
	 * 
	 */
	Action getAction(BoaState state);

}
